#encoding:utf-8
import os.path
import unittest

from ..image_compare import ImageCompare


class TestSIFTVLFeat(unittest.TestCase):

    def test_compare_equal(self):
        image1 = image2 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "bug.png")
        comparator = ImageCompare(image1, image2)

        compare = comparator.compare()

        self.assertAlmostEqual(compare, 1, places=2)

    def test_compare_rotation(self):
        image1 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "bug.png")
        image2 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "bug_90.png")
        comparator = ImageCompare(image1, image2)

        compare = comparator.compare()

        self.assertGreaterEqual(compare, 0.2)

    def test_compare_smaller(self):
        image1 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "bug.png")
        image2 = os.path.join(os.path.dirname(os.path.abspath(__file__)), "bug_smaller.png")
        comparator = ImageCompare(image1, image2)

        compare = comparator.compare()

        self.assertGreaterEqual(compare, 0.2)


if __name__ == '__main__':
    unittest.main()




import glob
import os
import subprocess

from PIL import Image
from numpy import *
from pylab import *


def process_image(imagename, resultname, params="--edge-thresh 10 --peak-thresh 5"):
    if imagename[-3:] != 'pgm':
        #create a pgm file
        im = Image.open(imagename).convert('L')
        imagename += ".pgm"
        im.save(imagename)

    cmmd = str("sift " + imagename + " --output=" + resultname + " " + params)
    print cmmd
    try:
        #process = subprocess.Popen(cmmd, shell=True)
        #process.communicate()
        os.system(cmmd)
    finally:
        files = glob.glob("/tmp/*.pgm")
        for fd in files:
            os.unlink(fd)


def read_features_from_file(filename):
    """ read feature properties and return in matrix form"""
    f = loadtxt(filename)
    return f[:, :4], f[:, 4:]  # feature locations, descriptors


def write_features_to_file(filename, locs, desc):
    """ save feature location and descriptor to file"""
    savetxt(filename, hstack((locs, desc)))


def plot_features(im, locs, circle=False):
    """ show image with features. input: im (image as array), 
        locs (row, col, scale, orientation of each feature) """

    def draw_circle(c, r):
        t = arange(0, 1.01, .01) * 2 * pi
        x = r * cos(t) + c[0]
        y = r * sin(t) + c[1]
        plot(x, y, 'b', linewidth=2)

    imshow(im)
    if circle:
        [draw_circle([p[0], p[1]], p[2]) for p in locs]
    else:
        plot(locs[:, 0], locs[:, 1], 'ob')
    axis('off')


def match(desc1, desc2):
    """
    for each descriptor in the first image,
    select its match in the second image.
    input: desc1 (descriptors for the first image),
    desc2 (same for second image).
    """

    desc1 = array([d / linalg.norm(d) for d in desc1])
    desc2 = array([d / linalg.norm(d) for d in desc2])

    dist_ratio = 0.6
    desc1_size = desc1.shape

    matchscores = zeros((desc1_size[0], 1))
    desc2t = desc2.T  # precompute matrix transpose
    for i in range(desc1_size[0]):
        dotprods = dot(desc1[i, :], desc2t)  # vector of dot products
        dotprods *= 0.9999
        #inverse cosine and sort, return index for features in second image
        indx = argsort(arccos(dotprods))

        #check if nearest neighbor has angle less than dist_ratio times 2nd
        if arccos(dotprods)[indx[0]] < dist_ratio * arccos(dotprods)[indx[1]]:
            matchscores[i] = int(indx[0])

    return matchscores


def appendimages(im1, im2):
    """ return a new image that appends the two images side-by-side."""

    # select the image with the fewest rows and fill in enough empty rows
    rows1 = im1.shape[0]
    rows2 = im2.shape[0]

    if rows1 < rows2:
        im1 = concatenate((im1, zeros((rows2 - rows1, im1.shape[1]))), axis=0)
    elif rows1 > rows2:
        im2 = concatenate((im2, zeros((rows1 - rows2, im2.shape[1]))), axis=0)
        #if none of these cases they are equal, no filling needed.

    return concatenate((im1, im2), axis=1)


def plot_matches(im1, im2, locs1, locs2, matchscores, show_below=True):
    """
    show a figure with lines joining the accepted matches
    input: im1,im2 (images as arrays), locs1,locs2 (location of features),
    matchscores (as output from 'match'), show_below (if images should be shown below).
    """

    im3 = appendimages(im1, im2)
    if show_below:
        im3 = vstack((im3, im3))

    # show image
    imshow(im3)

    # draw lines for matches
    cols1 = im1.shape[1]
    for i in range(len(matchscores)):
        if matchscores[i] > 0:
            plot([locs1[i, 0], locs2[matchscores[i, 0], 0] + cols1], [locs1[i, 1], locs2[matchscores[i, 0], 1]], 'c')
    axis('off')


def match_twosided(desc1, desc2):
    """ two-sided symmetric version of match(). """

    matches_12 = match(desc1, desc2)
    matches_21 = match(desc2, desc1)

    ndx_12 = matches_12.nonzero()[0]

    #remove matches that are not symmetric
    for n in ndx_12:
        if matches_21[int(matches_12[n])] != n:
            matches_12[n] = 0

    return matches_12



#encoding:utf-8
import os
from setuptools import setup, find_packages


PROJECT_PATH = os.path.dirname(os.path.abspath(__file__))

setup(
    name="sieve-sift",
    version="0.1.6",
    packages=find_packages(),
    scripts=["sift/sift", ],
    url="https://bitbucket.org/sievetech",
    license="",
    author="Sieve",
    author_email="sievetech@sieve.com.br",
    description="Sieve sift module.",
    data_files=[os.path.join(PROJECT_PATH, "sift", "sift"), ],
    install_requires=["matplotlib==1.1.1"]
)



#encoding:utf-8
import StringIO
import tempfile
import urllib


from PIL import Image
from numpy import ndarray

from sift import match, read_features_from_file, process_image


class ImageCompare:
    def __init__(self, image_path_1, image_path_2):
        self.image_path_1 = image_path_1
        self.image_path_2 = image_path_2

        self.loc1, self.desc1 = self.load_key_points(self.image_path_1)
        self.loc2, self.desc2 = self.load_key_points(self.image_path_2)

    def compare(self):
        """
        Compare two image and return the percentage of match
        """

        if isinstance(self.desc1, list):
            self.desc1 = ndarray.array(self.desc1)
        if isinstance(self.desc2, list):
            self.desc2 = ndarray.array(self.desc2)

        #Get the number of the keypoints in each image
        match_master = match(self.desc1, self.desc1)
        match_points_im1 = len(match_master)

        match_master = match(self.desc2, self.desc2)
        match_points_im2 = len(match_master)

        min_key_points = min(match_points_im1, match_points_im2)
        match_points = match(self.desc1, self.desc2)

        count_match_points = 0
        for point in match_points:
            if point[0] != 0:
                count_match_points += 1

        return float(count_match_points) / float(min_key_points)

    def load_key_points(self, image_source):
        """
        Load a image from a url, a local path or a file-like object and process
        the image using sift.

        @return: loc, description
        """
        with tempfile.NamedTemporaryFile() as imagefile, \
             tempfile.NamedTemporaryFile() as siftfile:
            try:
                if (isinstance(image_source, basestring) and
                        image_source.strip().startswith('http')):
                    fd = urllib.urlopen(image_source)
                    image = Image.open(StringIO.StringIO(fd.read()))

                else:
                    image = Image.open(image_source)
            except:
                return None, None

            # Store the images in B/W Int format
            image = image.convert("L")
            image.save(imagefile.name, "PPM")
            imagefile.flush()

            process_image(imagefile.name, siftfile.name)
            loc, description = read_features_from_file(siftfile.name)

            return loc, description
